using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Utils.DataBase;
using PhoenixDbAdmin.Class;
using Utils.Logs;

namespace PhoenixDbAdmin.GUI
{
    public partial class fr_Main : Form
    {
        #region Variables
        /// <summary>
        /// Report manager
        /// </summary>
        private ReportManager vReportManager = new ReportManager("PhoenixUserManag", "", "jibedoubleve.mine.nu", "Phoenix");
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        public fr_Main()
        {
            InitializeComponent();
            this.vReportManager.Updated += new EventHandler(Event_ReportManagerUpdated);
            try
            {
                dgv_Reports.DataSource = (vReportManager.SelectReport(false)).Tables[0];
                SetDataGridView();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Set the columns of the DataGridView
        /// </summary>
        private void SetDataGridView()
        {
            dgv_Reports.Columns["ID"].Visible = false;
            dgv_Reports.Columns["IDUser"].Visible = false;
            dgv_Reports.Columns["MEssage"].Visible = false;

            dgv_Reports.Columns["Date"].DisplayIndex = 0;

            dgv_Reports.Columns["Name"].HeaderText = "Name";
            dgv_Reports.Columns["Name"].DisplayIndex = 1;

            dgv_Reports.Columns["Surname"].HeaderText = "Surname";
            dgv_Reports.Columns["Surname"].DisplayIndex = 2;

            dgv_Reports.Columns["Type"].DisplayIndex = 3;

            dgv_Reports.Columns["Title"].HeaderText = "Title";
            dgv_Reports.Columns["Title"].DisplayIndex = 4;

            dgv_Reports.Columns["Message"].DisplayIndex = 5;

            dgv_Reports.Columns["IsRead"].Visible = false;
            dgv_Reports.Columns["IsRead"].HeaderText = "Lu?";
            dgv_Reports.Columns["IsRead"].DisplayIndex = 6;

        }
        #endregion

        #region Event
        /// <summary>
        /// Double click on DataGridView
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Click_DataGridView(object sender, EventArgs e)
        {
            if (dgv_Reports.Rows.Count <= 0) return;
            if (dgv_Reports.SelectedRows.Count <= 0) return;

            this.vReportManager.Date = (DateTime)dgv_Reports.SelectedRows[0].Cells["Date"].Value;
            this.vReportManager.ID = (int)dgv_Reports.SelectedRows[0].Cells["ID"].Value;
            this.vReportManager.IDUser = (int)dgv_Reports.SelectedRows[0].Cells["IDUser"].Value;

            object oIr = dgv_Reports.SelectedRows[0].Cells["IsRead"].Value;
            if (((int)((ulong)oIr)) == 0)
                this.vReportManager.IsRead = false;
            else
                this.vReportManager.IsRead = true;

            this.vReportManager.Message = (string)dgv_Reports.SelectedRows[0].Cells["Message"].Value;
            this.vReportManager.Title = (string)dgv_Reports.SelectedRows[0].Cells["Title"].Value;

            this.vReportManager.Type = (string)dgv_Reports.SelectedRows[0].Cells["Type"].Value;

            pg_Report.SelectedObject = this.vReportManager;
            tb_Message.Text = this.vReportManager.Message;
            this.gb_Message.Text = "Message (" + vReportManager.Title + ")";
        }
        /// <summary>
        /// Select unread reports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Unread(object sender, EventArgs e)
        {
            if (tsmi_Unread.Checked) return;
            tsmi_Unread.Checked = true;
            tsmi_Readed.Checked = false;
            try
            {
                dgv_Reports.DataSource = (this.vReportManager.SelectReport(false)).Tables[0];
                SetDataGridView();
                this.pg_Report.SelectedObject = null;
                this.tb_Message.Text = "";
                this.gb_Message.Text = "Message";
                this.Text = "Report administrator (Unread messages)";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// Select read reports
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_Readed(object sender, EventArgs e)
        {
            if (tsmi_Readed.Checked) return;
            tsmi_Readed.Checked = true;
            tsmi_Unread.Checked = false;
            try
            {
                dgv_Reports.DataSource = (this.vReportManager.SelectReport(true)).Tables[0];
                SetDataGridView();
                this.pg_Report.SelectedObject = null;
                this.tb_Message.Text = "";
                this.gb_Message.Text = "Message";
                this.Text = "Report administrator (Read messages)";
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Erreur", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        /// <summary>
        /// An options changed and it needs a database update
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Event_ReportManagerUpdated(object sender, EventArgs e)
        {
            if (tsmi_Readed.Checked)
                dgv_Reports.DataSource = (this.vReportManager.SelectReport(true)).Tables[0];
            else
                dgv_Reports.DataSource = (this.vReportManager.SelectReport(false)).Tables[0];

            this.pg_Report.SelectedObject = null;
            this.tb_Message.Text = "";
            this.gb_Message.Text = "Message";
        }
        /// <summary>
        /// Select all users
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuClick_AllUsers(object sender, EventArgs e)
        {
            try
            {
                fr_Users fr = new fr_Users();
                fr.DataSource = (this.vReportManager.GetUsers()).Tables[0];
                fr.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show("An error occured (" + ex.Message + ")", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Log.Instance.Write(ex, this, LogType.Error);
            }
        }
        #endregion
    }
}