using System;
using System.Collections.Generic;
using System.Text;
using Utils.DataBase;
using MySql.Data.MySqlClient;
using Utils.Logs;
using System.ComponentModel;
using Components.TypeEditors.GUI;
using System.Drawing.Design;
using Microsoft.Win32;
using System.Data;

namespace PhoenixUserManager.Class
{
    /// <summary>
    /// Manager for the inscription of a user
    /// </summary>
    public class UserManager
    {
        #region Variables
        /// <summary>
        /// ID of the user
        /// </summary>
        private int vUserID = -1;
        /// <summary>
        /// User's surname
        /// </summary>
        private string vSurname = "";
        /// <summary>
        /// User's name
        /// </summary>
        private string vName = "";
        /// <summary>
        /// User's mail
        /// </summary>
        private string vMail = "";
        /// <summary>
        /// User's date of inscription
        /// </summary>
        private DateTime vInscriptionDate = DateTime.Today;
        /// <summary>
        /// Reasons of the inscription
        /// </summary>
        private string vReasons = "";
        /// <summary>
        /// User's date of birth
        /// </summary>
        private DateTime vDateOfBirth = new DateTime();
        /// <summary>
        /// The SQL manager
        /// </summary>
        private MySQLDbSql SqlManager;
        /// <summary>
        /// Path of Registry for the software
        /// </summary>
        private string vSoftware = "";
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_Database">Database name</param>
        /// <param name="_Id">User's name</param>
        /// <param name="_Password">User's password</param>
        /// <param name="_Server">Server address</param>
        /// <param name="_Software">_Software name for the registry information</param>
        public UserManager(string _Id, string _Password, string _Server, string _Database, string _Software)
        {
            try
            {
                this.SqlManager = new MySQLDbSql(_Id, _Password, _Server, _Database);
                this.vSoftware += _Software;
            }
            catch (MySqlException) { throw; }
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_Database">Database name</param>
        /// <param name="_Id">User's name</param>
        /// <param name="_Password">User's password</param>
        /// <param name="_Server">Server address</param>        
        public UserManager(string _Id, string _Password, string _Server, string _Database)
        {
            this.SqlManager = new MySQLDbSql(_Id, _Password, _Server, _Database);
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_MySQLDbSql">MySQLDbSql SQL manager</param>
        /// <param name="_Software">Information for the regisrty</param>
        protected UserManager(MySQLDbSql _MySQLDbSql, string _Software)
        {
            this.SqlManager = _MySQLDbSql;
            this.vSoftware = _Software;
        }
        #endregion

        #region Methods
        /// <summary>
        /// Add the informations which are in the object un the central database
        /// </summary>
        /// <returns>The ID of the added user</returns>
        /// <exception cref="MySqlException">An exception occured in the database</exception>
        /// <exception cref="Exception">An unexpected exception occured</exception>
        /// <exception cref="SqlDataException">If the user does not have any Name or Surname</exception>
        public long Add()
        {
            string vStringInscriptionDate = this.InscriptionDate.Year + "-" + this.InscriptionDate.Month + "-" + this.InscriptionDate.Day;
            string vStringBirthDate = this.vDateOfBirth.Year + "-" + this.vDateOfBirth.Month + "-" + this.vDateOfBirth.Day;
            if (this.vName == "" || this.vSurname == "")
                throw new SqlDataException("Please select a name and/or a surname for the user!");
            if (this.vMail == null)
                this.vMail = "";
            this.Reasons = this.Reasons.Replace("\"", "");
            this.Reasons = this.Reasons.Replace("\\", "");
            SqlManager.SqlQuery = "INSERT INTO User (Name, Surname, Mail, DateInstall, Reasons, DateOfBirth) VALUES(\"" + this.Name + "\",\""
                + this.Surname + "\",\"" + this.Mail + "\", DATE(\'" + vStringInscriptionDate + "\'),\"" + this.vReasons + "\""
                + ", DATE(\'" + vStringBirthDate + "\'))";
            try
            {
                object ret = -1;
                SqlManager.ExecuteNonQuery(ref ret);                
                return (long)ret;
            }
            catch (SqlDataException) { throw; }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex.Message, this, LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex.Message, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Add the informations which are in the object un the central database
        /// </summary>
        /// <param name="_SetInRegistry">At TRUE the ID of inscription will be set in the registry 
        /// otherwize nothing in the registry</param>
        /// <returns>The ID of the added user</returns>
        /// <exception cref="MySqlException">An exception occured in the database</exception>
        /// <exception cref="Exception">An unexpected exception occured</exception>
        public long Add(bool _SetInRegistry)
        {
            try
            {
                if (!_SetInRegistry)
                    return Add();
                else
                {
                    long ID = Add();

                    #region Registry copy
                    if (this.vSoftware != null)
                    {
                        RegistryKey RKey = Registry.CurrentUser.OpenSubKey(this.vSoftware, true);
                        if (RKey == null)
                        {
                            RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true);
                            RKey.CreateSubKey(this.vSoftware);
                            RKey = RKey.OpenSubKey(this.vSoftware, true);
                        }
                        RKey.SetValue(@"Inscription", ID);
                    }
                    #endregion

                    return ID;
                }
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Fill the UserManager with the information found in the database
        /// </summary>
        /// <param name="_UserID">ID of the user</param>
        /// <returns>A filled DataSet</returns>
        /// <exception cref="Exception">If an error occured</exception>
        /// <exception cref="ArgumentException">Thrown if the ID does not exists in the user table</exception>
        public DataSet Fill(int _UserID)
        {
            try
            {
                SqlManager.SqlQuery = "SELECT * FROM User WHERE ID = " + _UserID + ";";
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);
                if (ds.Tables[0].Rows.Count <= 0)
                    throw new ArgumentException("No user exists with this Id!");
                this.InscriptionDate = (DateTime)ds.Tables[0].Rows[0]["DateInstall"];
                this.Mail = (string)ds.Tables[0].Rows[0]["Mail"];
                this.Name = (string)ds.Tables[0].Rows[0]["Name"];
                this.Surname = (string)ds.Tables[0].Rows[0]["Surname"];
                this.Reasons = (string)ds.Tables[0].Rows[0]["Reasons"];
                this.vUserID = _UserID;
                return ds;
            }
            catch (ArgumentException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Gets a DataSet with all the logs of the user represented bu _UserID
        /// </summary>
        /// <param name="_UserID">The user's ID</param>
        /// <returns>DataSet with all the log for this user</returns>
        /// <exception cref="Exception">If an error occured</exception>
        /// <exception cref="ArgumentException">Thrown if the ID does not exists in the user table</exception>
        public DataSet GetLogs(int _UserID)
        {
            try
            {
                SqlManager.SqlQuery = "SELECT * FROM Log WHERE IDUser = " + _UserID + ";";
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (ArgumentException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Add a report the the user given in the _UserID parameter
        /// </summary>
        /// <param name="_UserID">The id of the user who writes the report</param>
        /// <param name="_Message">Message of the report</param>
        /// <param name="_Title">Title of the report</param>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ArgumentException">The user ID does not exist in the database</exception>
        public void AddReport(int _ID, string _Title, string _Message)
        {
            try
            {
                if (Fill(_ID).Tables[0].Rows.Count <= 0)
                    throw new ArgumentException("L'identifiant de l'utilisateur n'existe pas dans la base de donnes!");

                string Date = DateTime.Now.Year + "-" + DateTime.Now.Month + "-" + DateTime.Now.Day + " "
                    + DateTime.Now.Hour + ":" + DateTime.Now.Minute + ":" + DateTime.Now.Second;
                _Message = _Message.Replace("\"", "");
                _Message = _Message.Replace("\'", "");
                _Message = _Message.Replace("\\", "");

                _Title = _Title.Replace("\'", "");
                _Title = _Title.Replace("\"", "");
                _Title = _Title.Replace("\\", "");

                this.SqlManager.SqlQuery = "INSERT INTO Log (Date, Type, Title, Message, IsRead, IDUser) VALUES "
                    + "(DATE(\'" + Date + "\'), \"" + LogType.Report + "\", \"" + _Title + "\",\"" + _Message
                    + "\", 0, " + _ID + ");";
                this.SqlManager.ExecuteNonQuery();
                Log.Instance.Write(_Message, _Title, LogType.Report);
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (ArgumentException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Check if the user with this Name and Surname exists
        /// </summary>
        /// <param name="_Name">Name of the user</param>
        /// <param name="_Surname">Surname of the user</param>
        /// <param name="_DateOfBirth">Date of birth of the user</param>
        /// <returns>TRUE if user exists otherwise FALSE</returns>
        public bool Exists(string _Name, string _Surname, DateTime _DateOfBirth)
        {
            DataSet ds = new DataSet();
            return Exists(_Name, _Surname, _DateOfBirth, ref ds);
        }
        /// <summary>
        /// Check if the user with this Name and Surname exists
        /// </summary>
        /// <param name="_Name">Name of the user</param>
        /// <param name="_Surname">Surname of the user</param>
        /// <param name="_ds">Contains the users with the _Name and _Surmane</param>
        /// <param name="_DateOfBirth">Date of birth of the user</param>
        /// <returns>TRUE if user exists otherwise FALSE</returns>
        public bool Exists(string _Name, string _Surname, DateTime _DateOfBirth, ref DataSet _ds)
        {
            string strDate = _DateOfBirth.Year + "-" + _DateOfBirth.Month + "-" + _DateOfBirth.Day;
            SqlManager.SqlQuery =
                "SELECT * FROM User WHERE Name LIKE \"" + _Name + "\" AND Surname LIKE \"" + _Surname
                + "\"AND DateOfBirth = DATE(\'" + strDate + "\');";
            _ds = new DataSet();
            int Count = SqlManager.ExecuteSql(ref _ds);
            if (Count == 0)
                return false;
            else
                return true;
        }
        /// <summary>
        /// Check if the user in this object is set in the Registry)
        /// </summary>        
        /// <returns>TRUE of the registry contains information about a user otherwise FALSE</returns>
        public bool RegistryContainsUser()
        {
            RegistryKey RKey = Registry.CurrentUser.OpenSubKey("Software");
            if (RKey == null)
                RKey = RKey.CreateSubKey("Software");

            RKey = RKey.OpenSubKey("UserInscription");
            if (RKey == null)
            {
                RKey = Registry.CurrentUser.OpenSubKey("Software", true);
                RKey = RKey.CreateSubKey("UserInscription");
            }

            RKey = RKey.OpenSubKey(vSoftware);
            if (RKey == null)
            {
                RKey = Registry.CurrentUser.OpenSubKey("Software", true).OpenSubKey("UserInscription", true);
                RKey = RKey.CreateSubKey(this.vSoftware);
                return false;
            }
            int _UserID;
            return Int32.TryParse((string)RKey.GetValue("Inscription"), out _UserID);
        }
        /// <summary>
        /// Gets the ID of the inscripted user 
        /// </summary>
        /// <returns>The user's ID</returns>        
        /// <exception cref="NullReferenceException">If there is no data in the registry</exception>
        public int GetIDInRegistry()
        {
            try
            {
                RegistryKey RKey = Registry.CurrentUser.OpenSubKey("Software").OpenSubKey("Inscription").OpenSubKey(this.vSoftware);
                if (RKey == null) throw new NullReferenceException("No user are written in the regisrty!");
                int i = (int)RKey.GetValue("User");
                return (int)RKey.GetValue("User");
            }            
            catch (NullReferenceException) { throw; }
        }
        /// <summary>
        /// Gets the ID of the inscripted user 
        /// </summary>
        /// <param name="_Software">The software registerd in the registry</param>
        /// <returns>The user's ID</returns>   
        /// <exception cref="NullReferenceException">If there is no data in the registry</exception>
        public static int GetIDInRegistry(string _Software)
        {
            try
            {
                RegistryKey RKey = Registry.CurrentUser.OpenSubKey("Software").OpenSubKey("Inscription").OpenSubKey(_Software);
                if (RKey == null) throw new NullReferenceException("No user are written in the regisrty!");
                return (int)RKey.GetValue("User");
            }            
            catch (NullReferenceException) { throw; }
        }
        /// <summary>
        /// Get an instance of UserManager with the user in the Registry
        /// </summary>
        /// <returns></returns>
        public UserManager GetRegistryUser()
        {
            if (!this.RegistryContainsUser())
                throw new Exception("The registry does not contains User information!");
            UserManager man = new UserManager(this.SqlManager, this.vSoftware);
            man.Fill(man.GetIDInRegistry());
            return man;
        }
        /// <summary>
        /// Set the registry with the information of this patient
        /// </summary>
        public void SetInRegistry()
        {
            try
            {
                if (vSoftware != null)
                {
                    RegistryKey RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true);
                    RKey = RKey.OpenSubKey(@"Inscription", true);
                    if (RKey != null)
                        RKey = RKey.OpenSubKey(vSoftware, true);
                    else
                    {
                        RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true);
                        RKey = RKey.CreateSubKey(@"Inscription");
                        RKey = RKey.CreateSubKey(vSoftware);
                    }
                    if (RKey == null)
                    {
                        RKey = Registry.CurrentUser.OpenSubKey(@"Inscription\", true);
                        RKey = RKey.CreateSubKey(vSoftware);
                        RKey = RKey.OpenSubKey(vSoftware, true);
                    }
                    RKey.SetValue(@"User", this.UserID);
                }
            }
            catch (NullReferenceException ex)
            {
                Log.Instance.Write(ex, typeof(UserManager), LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Set the registry with the information of this patient
        /// </summary>
        /// <param name="_ID">ID of the patient</param>
        /// <param name="_Software">Software name used to find the registry path</param>
        public static void SetInRegistry(int _ID, string _Software)
        {
            try
            {
                if (_Software != null)
                {
                    RegistryKey RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true);
                    RKey = RKey.OpenSubKey(@"Inscription", true);
                    if (RKey != null)
                    {
                        RKey = RKey.OpenSubKey(_Software, true);
                        if (RKey == null)
                        {
                            RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true).OpenSubKey("Inscription", true);
                            RKey = RKey.CreateSubKey(_Software);
                        }
                    }
                    else
                    {
                        RKey = Registry.CurrentUser.OpenSubKey(@"Software\", true);
                        RKey = RKey.CreateSubKey(@"Inscription");
                        RKey = RKey.CreateSubKey(_Software);
                    }
                    RKey.SetValue(@"User", _ID);
                }
            }
            catch (NullReferenceException ex)
            {
                Log.Instance.Write(ex, typeof(UserManager), LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Returns the name and the surname of the user
        /// </summary>
        /// <returns>Returns the name and the surname of the user</returns>
        public override string ToString()
        {
            return this.Name + " " + this.Surname;
        }
        /// <summary>
        /// Gets the logs of the user which is in the object
        /// </summary>
        /// <returns>DataSet with the logs</returns>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ArgumentException">If no user are set in the UserManager</exception>
        public DataSet GetLogs()
        {
            try
            {
                if (this.vUserID == -1)
                    throw new ArgumentException("No user are set in the UserManager!");
                DataSet ds = new DataSet();
                this.SqlManager.SqlQuery = "SELECT * FROM Log WHERE IDUser = " + this.vUserID + ";";
                this.SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (ArgumentException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets od sets the user's date of inscription
        /// </summary>
        [Browsable(false)]
        public DateTime InscriptionDate
        {
            get { return vInscriptionDate; }
            set { vInscriptionDate = value; }
        }
        /// <summary>
        /// Gets or sets the user's mail
        /// </summary>
        [Category("Inscription")]
        [Description("Mail")]
        [DisplayName("4- Mail")]
        public string Mail
        {
            get { return vMail; }
            set { vMail = value; }
        }
        /// <summary>
        /// Gets or sets the user's name
        /// </summary>
        [Category("Inscription")]
        [Description("Prnom")]
        [DisplayName("2- Prnom")]
        public string Name
        {
            get { return vName; }
            set { vName = value; }
        }
        /// <summary>
        /// Gets or sets the user's surname
        /// </summary>
        [Category("Inscription")]
        [Description("Nom de famille")]
        [DisplayName("1- Nom")]
        public string Surname
        {
            get { return vSurname; }
            set { vSurname = value; }
        }
        /// <summary>
        /// Gets or sets the date of birth of the user
        /// </summary>
        [Category("Inscription")]
        [Description("Date de naissance")]
        [DisplayName("3- Date de naissance")]
        public DateTime DateOfBirth
        {
            get { return vDateOfBirth; }
            set { vDateOfBirth = value; }
        }
        /// <summary>
        /// Gets or sets the reasons of the inscription
        /// </summary>
        [Category("Raisons")]
        [Description("Indiquez ici les raison du choix de ce logiciel.")]
        [DisplayName("1- Raison du choix")]
        [Editor(typeof(UILongTextEditor), typeof(UITypeEditor))]
        public string Reasons
        {
            get { return vReasons; }
            set { vReasons = value; }
        }
        /// <summary>
        /// Gets the ID of the user (Null if unset)
        /// </summary>
        [Browsable(false)]
        public int UserID { get { return this.vUserID; } }
        /// <summary>
        /// Gets the full name of the user
        /// </summary>
        [Browsable(false)]
        public string FullName { get { return this.ToString(); } }
        /// <summary>
        /// Gets the full name of the user and his date of birth
        /// </summary>
        [Browsable(false)]
        public string FullNameAndBirth { get { return this.ToString() + "(" + this.DateOfBirth + ")"; } }
        /// <summary>
        /// Gets the sofware name of the registry
        /// </summary>
        [Browsable(false)]
        public string Software
        {
            get { return vSoftware; }
            protected set { vSoftware = value; }
        }
        #endregion
    }
}
